@@ -927,7 +927,7 @@ class ConjectureRunner(object):
             except KeyError:
                 pass
             try:
-                b = min(b, self.capped[node_index])
+                b = b & self.capped[node_index]
             except KeyError:
                 pass
             try:
@@ -939,14 +939,15 @@ class ConjectureRunner(object):
 
     def cached_test_function(self, buffer):
         node_index = 0
-        for i in hrange(self.settings.buffer_size):
+        for c in buffer:
             try:
                 c = self.forced[node_index]
             except KeyError:
-                if i < len(buffer):
-                    c = buffer[i]
-                else:
-                    c = 0
+                pass
+            try:
+                c = c & self.capped[node_index]
+            except KeyError:
+                pass
             try:
                 node_index = self.tree[node_index][c]
             except KeyError:
